package indyjug.jboss.ejb;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

public class StockTraderBean implements SessionBean
{
    private SessionContext ctx;

    public Collection getStocks() throws NamingException, FinderException
    {
        try
        {
            StockLocalHome stockHome = StockUtil.getLocalHome();
            Collection stocks = stockHome.findAll();

            return getStockValues(stocks);
        }
        catch (NamingException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
        catch (FinderException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
    }

    public Collection getStocksByExchange(String exchange)
        throws NamingException, FinderException
    {
        try
        {
            StockLocalHome stockHome = StockUtil.getLocalHome();
            Collection stocks = stockHome.findByExchange(exchange);

            return getStockValues(stocks);
        }
        catch (NamingException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
        catch (FinderException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
    }

    public OrderValue trade(String exchange, String symbol, String action,
        double price, double shares)
        throws CreateException, NamingException, FinderException
    {
        try
        {
            StockLocalHome stockHome = StockUtil.getLocalHome();
            StockLocal stock = stockHome.findByPrimaryKey(symbol);

            OrderLocalHome orderHome = OrderUtil.getLocalHome();
            OrderLocal order = orderHome.create(stock, action, new Date(),
                    price, shares);

            return getOrderValue(order, getStockValue(stock));
        }
        catch (CreateException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
        catch (NamingException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
        catch (FinderException e)
        {
            ctx.setRollbackOnly();
            throw e;
        }
    }

    private Collection getStockValues(Collection originals)
    {
        Collection values = new ArrayList();
        StockValue value = null;

        for (Iterator iter = originals.iterator(); iter.hasNext();)
        {
            StockLocal local = (StockLocal)iter.next();
            value = getStockValue(local);
            values.add(value);
        }

        return values;
    }

    private StockValue getStockValue(StockLocal original)
    {
        StockValue value = new StockValue(original.getExchange(),
                original.getSymbol(), original.getName(), original.getPrice());

        return value;
    }

    private OrderValue getOrderValue(OrderLocal original, StockValue stock)
    {
        OrderValue value = new OrderValue(stock, original.getAction(),
                original.getDate(), original.getPrice(), original.getShares());

        return value;
    }

    public void ejbCreate() throws CreateException
    {
    }

    public void ejbRemove()
    {
    }

    public void ejbActivate()
    {
    }

    public void ejbPassivate()
    {
    }

    public void setSessionContext(SessionContext ctx)
    {
        this.ctx = ctx;
    }

    public void unsetSessionContext()
    {
        this.ctx = null;
    }
}
